/*
 * rawimg.h
 *
 * Copyright (C) 2012 Rhett Osborne
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef __RAWIMG_H__
 #define __RAWIMG_H__

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>

typedef enum
{
    raw_unset = 0,
    raw_i = 1,
    raw_ia = 2,
    raw_rgba = 3
}
raw_fmt;

typedef enum
{
    raw_bpp_unset = 0,
    raw_4bpp = 4,
    raw_8bpp = 8,
    raw_16bpp = 16,
    raw_32bpp = 32,
    __force_int16_alignment_bpp = INT16_MAX
}
raw_bpp;

/* valid combinations
     _________bpp_____
    |     |04|08|16|32|
    |rgba | p| p| i| x|
fmt |ia   | x| x| x|  |
    |i____|_x|_x|__|__|
 x = normal raw data
 i = pallete or normal
 p = pallete index
*/

typedef enum
{
    raw_pal_none = 0,
    raw_pal16 = 16,
    raw_pal256 = 256,
    __force_int16_alignment_pal = INT16_MAX
}
raw_pal;

typedef struct
{
    raw_fmt      fmt;
    union
    {
        raw_bpp  bpp;
        int16_t  __bpp;
    };
    struct
    {
        union
        {
            raw_pal  type;
            int16_t  __bpi;
        };
        int      entries;
        uint16_t *data;
    }
    pal;
    int          width;
    int          height;
    uint8_t     *data;
    union
    {
        void    *user_ptr;
        uint32_t user_uint;
        int      user_int;
    };
}
raw_image;

/* raw_png.c */
int raw_to_png( raw_image *r, char *png_name );
int png_to_raw( char *png_name, raw_image *r );
#if 0
/* raw_bmp.c */
int raw_to_bmp( raw_image *r, char *bmp_name );
int bmp_to_raw( char *bmp_name, raw_image *r );
#endif
/* rawimg.c */
int raw_to_raw( raw_image *in, raw_image *out ); /* out only needs fmt/bpp/pal.type set, sets & allocates feilds automatically */
int raw_valid_combo( raw_fmt fmt, raw_bpp bpp, raw_pal paltype );
int raw_get_size( raw_image *r ); /* size of r->data */
int raw_get_palsize( raw_image *r ); /* size of r->pal.data */
int raw_get_maxpalentries( raw_image *r ); /* 0, 16, or 256 */
char * raw_gen_name( raw_image *r, char *dst, size_t siz );
void raw_clean( raw_image *r );
int raw_alloc_structures( raw_image *r );
int name_gen_raw( char *name, raw_image *r );

#endif /* __RAWIMG_H__ */

